<?php

namespace App\Http\Controllers\Api\mini;

use App\Models\Device;
use App\Models\DeviceName;
use App\Models\InnerDevice;
use App\Models\Option;
use App\Models\Order;
use App\Models\OrderDevice;
use App\Models\Project;
use App\Models\ProjectRole;
use App\Models\ProjectUser;
use App\Models\RentType;
use App\Models\Spec;
use App\Models\WorkPoint;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Log;

class OrderController extends BaseController
{
    protected $model;

    public function __construct()
    {
        $this->model = new Order();
    }

    public function create(Request $request)
    {
        $project = Project::find($request->input('project_id'));
        if(!$project) return $this->error(['msg' => '找不到项目']);
        $user = Auth::guard('mini')->user();
        $project_role_id = $request->input('is_draft') == 1 ? ProjectRole::getByKey('work', 'id') : ProjectRole::getByKey('machine', 'id');
        $option = Option::get('orders', 'status', 'checking');

        $order = Order::create([
            'work_point_id' => $request->input('work_point_id'),
            'remark' => $request->input('remark'),
            'is_draft' => $request->input('is_draft'),
            'status' => $option,
            'order_number' => $this->model->createOrderNumber(),
            'project_id' => $project->id,
            'user_id' => $user->id,
            'project_role_id' => $project_role_id,
            'type' => 1
        ]);
        if(!$order) return $this->error(['msg' => '订单创建失败']);
        $devices = $request->input('devices');
        $total = 0;
        foreach($devices as $device) {
            $price = $this->transMoney($device['price']);
            $data = $this->getNameSpecRent($device);
            OrderDevice::create(array_merge([
                'order_id' => $order->id,
                'project_id' => $order->project_id,
                'quantity' => $device['quantity'],
                'price' => $price,
                'start_date' => $device['start_date'],
                'end_date' => $device['end_date']
            ], $data));
            $total = $total + ($price * (int)$device['quantity']);
        }
        $order->update(['money' => $total]);
        return $this->success();
    }

    public function getNameSpecRent($data)
    {
        $device = Device::find($data['type_id']);

        $rent_type = RentType::firstOrCreate([
            'name' => $data['rent']
        ]);
        if(!$device) return [
            'device_id' => '',
            'device_name_id' => '',
            'spec_id' => '',
            'rent_type_id' => $rent_type->id
        ];
        $device_name = DeviceName::firstOrCreate([
            'device_id' => $device->id,
            'name' => $data['name']
        ]);
        $spec = Spec::firstOrCreate([
            'device_id' => $device->id,
            'device_name_id' => $device_name->id,
            'name' => $data['spec']
        ]);
        return [
            'device_id' => $device->id,
            'device_name_id' => $device_name->id,
            'spec_id' => $spec->id,
            'rent_type_id' => $rent_type->id
        ];
    }

    public function createInner(Request $request)
    {
        $project = Project::find($request->input('project_id'));
        if(!$project) return $this->error(['msg' => '找不到项目']);
        $devices = $request->input('devices');
        $using_id = Option::get('inner_devices', 'status', 'using');
        $free_id = Option::get('inner_devices', 'status', 'free');

        // 续租时不检查设备使用状态
        if(!$request->input('type') || $request->input('type') != 're-rent') {
            foreach($devices as $device) {
                $device = InnerDevice::find($device['id']);
                if($device->status != $free_id) {
                    $status = Option::find($device->status);
                    $msg = $device['name'] . ($status ? $status['name'] : '使用中') . '，不能使用';
                    return $this->error(['msg' => $msg]);
                }
            }
        }

        $user = Auth::guard('mini')->user();
        $project_role_id = $request->input('is_draft') == 1 ? ProjectRole::getByKey('machine', 'id') : ProjectRole::getByKey('assist', 'id');
        $option = Option::get('orders', 'status', 'checking');

        $order = Order::create([
            'work_point_id' => $request->input('work_point_id'),
            'remark' => $request->input('remark'),
            'is_draft' => $request->input('is_draft'),
            'status' => $option,
            'order_number' => $this->model->createOrderNumber(),
            'project_id' => $project->id,
            'user_id' => $user->id,
            'project_role_id' => $project_role_id,
            'type' => 2
        ]);
        if(!$order) return $this->error(['msg' => '订单创建失败']);
        foreach($devices as $device) {
            OrderDevice::create([
                'name' => $device['name'],
                'order_id' => $order->id,
                'project_id' => $order->project_id,
                'inner_device_id' => $device['id'],
                'start_date' => $device['start_date'],
                'end_date' => $device['end_date']
            ]);
            if($device['id']) {
                InnerDevice::find($device['id'])->update([
                    'project_id' => $order['project_id'],
                    'status' => $using_id,
                    'start_date' => $device['start_date'],
                    'end_date' => $device['end_date'],
                    'work_point_id' => $request->input('work_point_id')
                ]);
            }
        }

        return $this->success();
    }

    public function update(Request $request)
    {
        $order = $this->model->find($request->input('id'));
        if(!$order) return $this->error(['msg' => '找不到订单']);
        $user = Auth::guard('mini')->user();
        $project_role_id = $request->input('is_draft') == 1 ? ProjectRole::getByKey('work', 'id') : ProjectRole::getByKey('machine', 'id');
        $option = Option::get('orders', 'status', 'checking');
        $res = $order->update([
            'work_point_id' => $request->input('work_point_id'),
            'remark' => $request->input('remark'),
            'is_draft' => $request->input('is_draft'),
            'status' => $option,
            'order_number' => $this->model->createOrderNumber(),
            'user_id' => $user->id,
            'project_role_id' => $project_role_id
        ]);
        if(!$res) return $this->error(['msg' => '订单修改失败']);
        $devices = $request->input('devices');
        $total = 0;
        OrderDevice::where('order_id', '=', $order->id)->delete();
        foreach($devices as $device) {
            $price = $this->transMoney($device['price']);
            OrderDevice::create([
                'name' => $device['name'],
                'order_id' => $order->id,
                'device_id' => $device['type_id'],
                'quantity' => $device['quantity'],
                'price' => $price,
                'start_date' => $device['start_date'],
                'end_date' => $device['end_date']
            ]);
            $total = $total + ($price * (int)$device['quantity']);
        }
        $order->update(['money' => $total]);
        return $this->success();
    }

    public function updateInner(Request $request)
    {
        $order = $this->model->find($request->input('id'));
        if(!$order) return $this->error(['msg' => '找不到订单']);
        $user = Auth::guard('mini')->user();
        $project_role_id = $request->input('is_draft') == 1 ? ProjectRole::getByKey('work', 'id') : ProjectRole::getByKey('machine', 'id');
        $type = $request->input('type');
        $option = Option::get('orders', 'status', 'checking');
        if($type == 'back') {
            $option = Option::get('orders', 'status', 'back');
        }

        $res = $order->update([
            'work_point_id' => $request->input('work_point_id'),
            'remark' => $request->input('remark'),
            'is_draft' => $request->input('is_draft'),
            'status' => $option,
            'user_id' => $user->id,
            'project_role_id' => $project_role_id
        ]);
        if(!$res) return $this->error(['msg' => '订单修改失败']);
        $devices = $request->input('devices');
        if($type == 'back') {
            foreach($devices as $device) {
                if($device['id']) {
                    InnerDevice::find($device['id'])->update([
                        'project_id' => '',
                        'start_date' => '',
                        'end_date' => '',
                        'work_point_id' => '',
                        'status' => Option::get('inner_devices', 'status', 'free')
                    ]);
                }
            }
        } else {
            OrderDevice::where('order_id', '=', $order->id)->delete();
            foreach($devices as $device) {
                OrderDevice::create([
                    'name' => $device['name'],
                    'order_id' => $order->id,
                    'device_id' => $device['id'],
                    'start_date' => $device['start_date'],
                    'end_date' => $device['end_date']
                ]);
                if($device['id']) {
                    InnerDevice::find($device['id'])->update([
                        'start_date' => $device['start_date'],
                        'end_date' => $device['end_date'],
                        'work_point_id' => $request->input('work_point_id')
                    ]);
                }
            }
        }

        return $this->success();
    }

    public function get(Request $request)
    {
        $items = $this->model->join('users', 'orders.user_id', '=', 'users.id')->select('orders.*', 'users.id as user_id', 'users.name');
        $search_items = [
            ['project_id', $request->input('project_id')]
        ];
        $is_draft = $request->input('is_draft');
        if($is_draft) {
            $search_items['is_draft'] = $is_draft;
        }
        if($request->input('status')) {
            $status = Option::get('orders', 'status', $request->input('status'));
            if($status) {
                array_push($search_items, ['status', '=', $status]);
            };
        }
        $equal_items = ['work_point_id', 'type'];
        foreach ($equal_items as $item) {
            if($request->input($item)) {
                array_push($search_items, [$item, '=', $request->input($item)]);
            }
        }
        $items = $items->where($search_items);
        if($request->input('keyword')) {
            $keyword = '%' . $request->input('keyword') . '%';
            $items = $items->where(function ($query) use($keyword) {
                $query->where('order_number', 'like', $keyword)->orWhere('users.name', 'like', $keyword);
            });
        }
        $items = $items->orderBy('updated_at', 'desc');
        $items = $items->paginate();
        foreach($items as $item) {
            $item->formatOrder($item);
        }
        return $this->success(['data' => $items->items()]);
    }


    public function formatOrder($item)
    {
        $option = Option::find($item->status);
        $item->status = $item->getStatusName();
        $item->status_key = $option ? $option['key'] : '';
        $item->color = $option ? $option['color'] : '';
        $item->date_time = substr($item->created_at, 0, 16);
    }

    public function detail(Request $request)
    {
        $order = $this->model->with('project', 'workPoint', 'devices', 'user', 'innerDevices')->find($request->input('id'));
        if(!$order) return $this->error(['msg' => '找不到订单']);
        $this->formatOrder($order);
        foreach($order['innerDevices'] as $device) {
            $device->status = Option::find($device->status);
            $device->spec = Spec::find($device->spec);
            $device->work_point = WorkPoint::find($device->work_point_id);
        }
        return $this->success(['data' => $order]);
    }

    public function getRole(Request $request)
    {
        $user = Auth::guard('mini')->user();
        $order = $this->model->find($request->input('id'));
        if(!$order) return $this->error(['msg' => '找不到订单']);
        $project_user = ProjectUser::where([
            ['project_id', '=', $order->project_id],
            ['user_id', '=', $user->id]
        ])->with('projectRole')->first();
        return $this->success(['data' => $project_user]);
    }

    public function check(Request $request)
    {
        $user = Auth::guard('mini')->user();
        $order = $this->model->find($request->input('id'));
        if(!$order) return $this->error(['msg' => '找不到订单']);
        $project_user = ProjectUser::where([
            ['project_id', '=', $order->project_id],
            ['user_id', '=', $user->id]
        ])->first();
        if(!$project_user) return $this->error(['msg' => '找不到用户']);
        $project_role = ProjectRole::find($project_user->project_role_id);
        if(!$project_role) return $this->error(['msg' => '找不到角色']);
        $status = Option::get('orders', 'status', $request->input('type'));

        $is_inner = $order->type == 2;
        // 工区负责人或机电负责人确认通过
        if($request->input('type') == 'confirm') {
            $status = Option::get('orders', 'status', 'pass');
            $next_project_role_id = $project_role->id;
        } else if($request->input('type') == 're-submit') {
            $status = Option::get('orders', 'status', 'checking');
            $next_project_role_id = $project_role->getNext('id', $is_inner);
        } else if($request->input('type') == 'reject') {
            if($is_inner) {
                $next_project_role_id = ProjectRole::getByKey('machine', 'id');
            } else {
                $next_project_role_id = ProjectRole::getByKey('work', 'id');
            }
        } else {
            $next_project_role_id = $project_role->getNext('id', $is_inner);
        }

        $res = $order->update([
            'status' => $status,
            'last_project_role_id' => $project_role->id,
            'project_role_id' => $next_project_role_id
        ]);
        if($res) return $this->success();
        return $this->error(['msg' => '操作失败']);
    }

    public function changePrice(Request $request)
    {
        $order_device = OrderDevice::find($request->input('id'));
        if(!$order_device) return $this->error(['msg' => '找不到设备']);
        $order_device->update([
            'price' => $this->transMoney($request->input('price'))
        ]);
        $order = $this->model->find($order_device['order_id']);
        if(!$order) return $this->error(['msg' => '找不到订单']);
        $order->updateMoney();
        return $this->success();
    }
}
